package ru.dbotthepony.mc.otm.client.render

import net.minecraft.resources.ResourceLocation

class SubSkinElement(
	val parent: AbstractSkinElement,
	val xOffset: Float = 0f,
	val yOffset: Float = 0f,
	val subWidth: Float = parent.width,
	val subHeight: Float = parent.height,
	private val overrideWinding: UVWindingOrder? = null
) : AbstractSkinElement() {
	override val width: Float
		get() = subWidth
	override val height: Float
		get() = subHeight

	override val u0: Float
		get() = parent.u0 + (xOffset / parent.width) * (parent.u1 - parent.u0)
	override val v0: Float
		get() = parent.v0 + (yOffset / parent.height) * (parent.v1 - parent.v0)

	override val u1: Float
		get() = parent.u0 + ((xOffset + subWidth) / parent.width) * (parent.u1 - parent.u0)
	override val v1: Float
		get() = parent.v0 + ((yOffset + subHeight) / parent.height) * (parent.v1 - parent.v0)

	override val texture: ResourceLocation
		get() = parent.texture

	override val type: SkinElementType
		get() = SkinElementType.SUBELEMENT

	override val winding: UVWindingOrder
		get() = overrideWinding ?: parent.winding

	fun copy(
		xOffset: Float = this.xOffset,
		yOffset: Float = this.yOffset,
		subWidth: Float = this.subWidth,
		subHeight: Float = this.subHeight,
		overrideWinding: UVWindingOrder? = this.overrideWinding
	) = SubSkinElement(parent, xOffset, yOffset, subWidth, subHeight, overrideWinding)
}

fun AbstractSkinElement.subElement(
	x: Float = 0f,
	y: Float = 0f,
	width: Float = this.width,
	height: Float = this.height,
) = SubSkinElement(this, x, y, width, height)
